package org.rcisoft.integration.aep.service.impl;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.integration.aep.service.IAepPushService;
import org.rcisoft.integration.video.Service.IDaHuaVideoService;
import org.rcisoft.sys.doordeviceinfo.entity.DoorDeviceInfo;
import org.rcisoft.sys.doordeviceinfo.service.DoorDeviceInfoService;
import org.rcisoft.sys.doorreportdata.entity.DoorReportData;
import org.rcisoft.sys.doorreportdata.service.DoorReportDataService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;

/**
 * AEP门磁设备推送处理逻辑
 */
@Service
@Slf4j
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
public class AepPushServiceImpl  implements IAepPushService {

    @Autowired
    private DoorDeviceInfoService doorDeviceInfoService;

    @Autowired
    private DoorReportDataService doorReportDataService;

    @Autowired
    private IDaHuaVideoService daHuaVideoService;

    @Value("${dahua.videoStatus}")
    private boolean videoStatus;

    /**
     * 门磁数据上报处理
     * @param data
     */
    @Async
    @Override
    public void saveDoorSensorData(JSONObject data) {
        String msgType = data.getString("messageType");
        switch (msgType){
            case "dataReport": //数据上报
                dataReportHandle(data);
                break;
            case "eventReport": // 事件上报
                eventReportHandle(data);
                break;
            case "deviceOnlineOfflineReport"://设备上下线
                deviceUpOrDownHandle(data);
                break;
            case "addDeleteDevice": //设备添加删除
                deviceAddOrDelHandle(data);
                break;
        }
    }

    /**
     * 设备数据上报处理逻辑
     * @param data
     */
    private void dataReportHandle(JSONObject data){
        //获取设备编码
        String deviceIMEICode = data.getString("IMEI");
        //获取上报时间
        long timestamp = data.getLongValue("timestamp");
        //获取设备负载信息
        JSONObject payload = data.getJSONObject("payload");
        if(payload!=null && payload.containsKey("APPdata")){
            String deviceData = payload.getString("APPdata");
            //Base64解码
            String hexStr = Base64.decodeStr(deviceData);
            DoorReportData reportData = new DoorReportData();
            reportData.setDeviceImei(deviceIMEICode);
            String dataStatus = convertBean(reportData, hexStr);
            if(StringUtils.equals("200",dataStatus)){
                DoorDeviceInfo deviceInfo = doorDeviceInfoService.getDeviceInfoByIMEICode(deviceIMEICode);
                if(deviceInfo!=null){
                    reportData.setDeviceId(Long.valueOf(deviceInfo.getBusinessId()));
                }
                reportData.setReportTime(new Date(timestamp));
                reportData.setFlag("0");
                reportData.setDelFlag("0");
                //保存数据操作
                doorReportDataService.persist(reportData);
                if(videoStatus && deviceInfo.getLinkVideoId()!=null && StringUtils.equals("OPEN",reportData.getDataType())){
                    daHuaVideoService.grabVideoDevicePicData(deviceInfo.getLinkVideoId(),reportData);
                }
            }
        }else{
            //异常信息，获取负载信息失败了
            log.error("当前设备["+deviceIMEICode+"] 上报数据内容为空，属于异常数据");
        }
    }

    /**
     * 根据Hex解析数据并返回对象
     * @param hexStr
     * @return 解析状态
     *  -1: 解析失败，数据存在异常
     *  200：解析正常
     *  0：解析正常，数据无需处理
     */
    private String convertBean(DoorReportData convertBean,String hexStr){
        /**
         * 设备上报数据： AA55 01 03 0A 010000000000280C0000 43
         * 设备心跳：    AA55 01 02 14 010000000000530C53649C004E80AD0011000013 69
         * 设备注册：    AA55 01 06 18 011001046011311883631901101089861120224015401557 72
         *
         */
        String resultCode =null;
        try{
            if(hexStr.length()>=32){
                String frameHeaderStr = hexStr.substring(0,4);//帧头
                String deviceTypeStr = hexStr.substring(4,6);// 产品类型
                String commandStr = hexStr.substring(6,8);//指令类型
                String dataLengthStr = hexStr.substring(8,10);//数据包长度
                int dataLength = Integer.parseInt(dataLengthStr,16);
                String payloadData = hexStr.substring(10,(10+dataLength*2));//数据包内容
                if(StringUtils.equals("AA55",frameHeaderStr)){
                    switch (commandStr){
                        case "01"://设备注册
                        case "06": //设备注册新
                        case "05": //NB网关报警
                        case "02": //心跳数据
                            resultCode = "0";
                            log.info("设备：【"+convertBean.getDeviceImei()+"】发送的指令：【"+hexStr+"】 无需入库");
                            break;
                        case "03": //NB单品报警
                            //开始解析负载数据信息
                            String status = payloadData.substring(0, 4);//门磁状态
                            String deviceStatus = payloadData.substring(4,12);//设备状态
                            String fcStatus = deviceStatus.substring(4, 5);//防拆状态
                            String dcStatus = deviceStatus.substring(6, 7);//电池状态
                            if("1".equals(fcStatus) || "1".equals(dcStatus)){//发生报警信息，不再检查门状态
                                convertBean.setDataType("WARN");
                                convertBean.setDataVal("1".equals(fcStatus)?"设备被拆，请检查":"设备电量低，请检查");
                            }else{
                                if(StringUtils.equals("0100",status)){
                                    convertBean.setDataType("OPEN");//开门状态
                                }else if(StringUtils.equals("0000",status)){
                                    convertBean.setDataType("CLOSE");//关门
                                }else {
                                    resultCode = "0";
                                    log.info("设备：【"+convertBean.getDeviceImei()+"】发送的指令：【"+hexStr+"】 无需入库");
                                    break;
                                }
                            }
                            convertBean.setRemarks(hexStr);
                            resultCode = "200";
                            break;
                    }
                }else{
                    //指令不符合要求，属于错误数据
                    log.error("设备：【"+convertBean.getDeviceImei()+"】发送的指令：【"+hexStr+"】 错误，无法解析");
                    return "-1";
                }

            }else{
                //指令不符合要求，属于错误数据
                log.error("设备：【"+convertBean.getDeviceImei()+"】发送的指令：【"+hexStr+"】 错误，无法解析");
                resultCode = "-1";
            }
        }catch (Exception e){
            log.error(e.getMessage(),e);
            log.error("设备：【"+convertBean.getDeviceImei()+"】发送的指令：【"+hexStr+"】 解析错误");
            resultCode = "-1";
        }
        return resultCode;
    }



    /**
     * 设备事件上报处理逻辑
     * @param data
     */
    private void eventReportHandle(JSONObject data){
        //获取设备编码
        String deviceIMEICode = data.getString("IMEI");
        //获取上报时间
        long timestamp = data.getLongValue("timestamp");
        //事件类型 1：信息 2：警告 3： 故障
        String eventType = data.getString("eventType");
        // 事件内容 暂不解析
        JSONObject eventData = data.getJSONObject("eventContent");
        if(StringUtils.equals("2",eventType) || StringUtils.equals("3",eventType)){
            DoorReportData reportData = new DoorReportData();
            reportData.setDeviceImei(deviceIMEICode);
            reportData.setDataType("WARN");
            reportData.setFlag("0");
            reportData.setDelFlag("0");
            reportData.setReportTime(new Date(timestamp));
            DoorDeviceInfo deviceInfo = doorDeviceInfoService.getDeviceInfoByIMEICode(deviceIMEICode);
            if(deviceInfo!=null){
                reportData.setDeviceId(Long.valueOf(deviceInfo.getBusinessId()));
            }
            StringBuffer dataValStr = new StringBuffer();
            dataValStr.append("设备发生【").append(StringUtils.equals("2",eventType)?"警告":"故障").append("】事件");
            dataValStr.append(" 事件内容：【").append(eventData.toJSONString()).append("】");
            reportData.setDataVal(dataValStr.toString());
            doorReportDataService.persist(reportData);
        }else{
            log.error("设备：【"+deviceIMEICode+"】 上报设备事件信息："+eventData.toJSONString());
        }

    }

    /**
     * 设备上下线处理逻辑
     * 此处没有设备的编码数据，无法入库处理，因此此处仅做日志记录
     * @param data
     */
    private void deviceUpOrDownHandle(JSONObject data){
        log.info("系统设备上线/下线通知：【"+data.toJSONString()+"】");
    }


    /**
     * 设备添加删除处理逻辑
     * 设备添加删除逻辑不再系统内体现，因此此处可能就没有上报的内容，但是为保证消息完整记录，此处记录下来
     * @param data
     */
    private void deviceAddOrDelHandle(JSONObject data){
        log.info("系统设备发生变更：【"+data.toJSONString()+"】");
    }
}
